from git_upm_publisher.utils.config_reader import Config
from git_upm_publisher.utils.git_manager import Git
from git_upm_publisher.utils.package_manager import PackageManager


def main():
    config = Config()
    git = Git(config.repo_root_path())
    pm = PackageManager(config.package_root_path())

    package = pm.read()
    package_version = package.get("version")
    print("Version from package.json: " + package_version)
    version_tag = package_version  # + "-upm"
    print("Version tag: " + version_tag)

    print(git.status())
    should_push = input("Fetch before starting? (y/n): ")

    if should_push == 'y':
        git.fetch()
        print(git.status())

    if git.is_dirty():
        print("You have outstanding changes in your repository.")
        print("You have a couple of options:")
        print("\t0. Cancel and deal with them manually (RECOMMENDED)")
        print("\t1. Commit -> publish -> soft reset (will try to preserve changes)")
        print("\t2. Clean -> publish (will erase all changes)")
        opt = input("Pick an index: ")

        if opt == '0':
            raise Exception("Cancelled by user.")
        elif opt == '1':
            temp_commit_message = "!!! TEMP COMMIT CREATED BY git-upm-publisher, WILL BE SOFT RESET !!!\n\nThis commit should be gone after everything is done, if you see it in your repository after publisher is done, something went wrong. Send me a bug report over GitHub.\n\nYou can manually soft-reset this commit if you wish."
            git.commit_all(temp_commit_message)
            git.publish(config.package_root_path(), "upm release", "upm", version_tag)
            git.soft_reset_last_commit()
        elif opt == '2':
            git.clean()
            git.publish(config.package_root_path(), "upm release", "upm", version_tag)
        else:
            raise Exception("Wrong option index.")
    else:
        git.publish(config.package_root_path(), "upm release", "upm", version_tag)

    print(git.status())
    should_push = input("Push everything? (y/n): ")

    if should_push == 'y':
        git.push_all()
        print(git.status())


if __name__ == '__main__':
    main()
    input("Press any key to exit.")



import os
import json
import jsonpickle
from git_upm_publisher.utils import discover
from git_upm_publisher.utils.package_json_obj import PackageJsonObj


class PackageManager:
    def __init__(self, package_root_path: str):
        if not os.path.exists(package_root_path):
            print("Warning: Package root does not exist.")

        self.package_root_path = package_root_path
        self.package_json_path = os.path.join(package_root_path, "package.json")

    def exists(self):
        return discover.package_json_exist_in_directory(self.package_root_path)

    @staticmethod
    def get_dependencies():
        print("Dependencies (one per line, blank to terminate):")
        inputs = {}

        while True:
            last_input = input("> ")
            if last_input == "":
                break
            else:
                parsed = last_input.split(":")
                inputs[parsed[0]] = parsed[1]

        return inputs

    def create(self):
        package_name = input("Package name: ")
        display_name = input("Display name: ")
        unity_min_version = input("Unity min version (blank if doesn't apply): ")
        description = input("Description: ")
        version = input("Version: ")
        dependencies = self.get_dependencies()

        if not os.path.exists(self.package_root_path):
            os.makedirs(self.package_root_path)

        json_obj = PackageJsonObj()
        json_obj.init_with_values(package_name, display_name, unity_min_version, description, version, dependencies)
        self.save(json_obj)

    def refresh(self):
        self.read()

    def read(self):
        with open(self.package_json_path, "r") as fp:
            try:
                json_decoded = jsonpickle.decode(fp.read())
            except Exception as ex:
                raise Exception("package.json file is not valid JSON (" + str(ex) + ")")
            json_obj = PackageJsonObj()
            json_obj.init_from_dict(json_decoded)
            return json_obj

    def save(self, json_obj: PackageJsonObj):
        with open(self.package_json_path, "w+") as fp:
            fp.write(json.dumps(json.loads(jsonpickle.encode(json_obj, unpicklable=False)), indent=4, sort_keys=True))



from git_upm_publisher.utils.config_reader import Config
from git_upm_publisher.utils.package_manager import PackageManager


def main():
    config = Config()
    pm = PackageManager(config.package_root_path())

    if not pm.exists():
        raise Exception("Cannot locate package.json")

    json_obj = pm.read()

    print("Enter the key of the property to change, one per line. 'c' to cancel. 's' to save.")

    while True:
        property_to_change = input("> ")

        if property_to_change == "c":
            raise Exception("Cancelled by user")
        elif property_to_change == "s":
            break

        try:
            current_value = json_obj.get(property_to_change)
            print("Current value: " + str(current_value))
        except:
            print("Key not found.")
            continue

        new_value = input("New value: ")
        json_obj.set(property_to_change, new_value)

    pm.save(json_obj)
    print("Done.")


if __name__ == '__main__':
    main()
    input("Press any key to exit.")



from pathlib import Path
import os
from tkinter import filedialog
from tkinter import *

tkroot = Tk()
tkroot.withdraw()


def pattern(pattern_, path):
    for filename in Path(path).glob(pattern_):
        print(filename)
        yield filename


def pattern_recursive(pattern_, path):
    return pattern('**/' + pattern_, path)


def package_json_recursive(path):
    return pattern_recursive('package.json', path)


def directory_of(path):
    p = Path(path)
    if p.is_dir():
        return p
    elif p.is_file():
        return p.parent


def directory_of_file(file):
    p = Path(file)
    assert p.is_file(), 'Path is not a file.'
    return p.parent


def make_relative(path, anchor):
    if path is not Path:
        path = Path(path).absolute()
    if anchor is not Path:
        anchor = Path(anchor).absolute()

    try:
        return Path(os.path.relpath(path, anchor))
    except:
        print("Error while trying to make the path relative. Returning absolute path.")
        return path.absolute()


def ask_directory(title, initialdir, mustexist):
    return filedialog.askdirectory(title=title, initialdir=initialdir, mustexist=mustexist)


def ask_repo_root():
    # repo_root_path = input("Repo root path (can be relative): ")
    return ask_directory("Select the root directory of the Git Repository...", '.', True)


def ask_package_root(initialdir):
    # package_root_path = input("Package root path (can be relative): ")
    return ask_directory("Select the root directory of UPM package...", initialdir, True)


def file_exist_in_directory(directory, filename):
    found = pattern(filename, directory)
    return len(list(found)) > 0


def package_json_exist_in_directory(directory):
    return file_exist_in_directory(directory, "package.json")



import warnings
warnings.warn("This package is deprecated in favor of [git-upm-publisher-2](https://github.com/starikcetin/git-upm-publisher-2).", DeprecationWarning)

import os
from git_upm_publisher.utils.config_reader import Config
from git_upm_publisher.utils.package_manager import PackageManager
from git_upm_publisher import config_maker
from git_upm_publisher import package_json_creator
from git_upm_publisher import package_json_updater
from git_upm_publisher import upm_publisher


def run_config_maker():
    if os.path.exists("config.json"):
        overwrite_config = input("You already have a config.json file. Do you want to reconfigure it? (y/n): ")
        if overwrite_config != 'y':
            return
    print("Starting config_maker")
    config_maker.main()


def run_package_makers(package: PackageManager):
    if package.exists():
        package.read()
        print("You already have a package.json file. You can...")
        print("0. Use it as is")
        print("1. Recreate it completely")
        print("2. Update it partially")
        opt = int(input("Enter the index of your choice: "))
        if opt == 0:
            return
        if opt == 1:
            print("Starting package_json_creator")
            package_json_creator.main()
        if opt == 2:
            print("Starting package_json_updater")
            package_json_updater.main()
    else:
        print("Starting package_json_creator")
        package_json_creator.main()


def run_upm_publisher():
    print("Starting upm_publisher")
    upm_publisher.main()


def main():
    config = Config()

    run_config_maker()
    config.refresh()
    package = PackageManager(config.package_root_path())

    run_package_makers(package)
    package.refresh()

    run_upm_publisher()


if __name__ == '__main__':
    main()
    input("Press any key to exit.")



from git_upm_publisher.utils.config_reader import Config
from git_upm_publisher.utils import discover


def main():
    if len(list(discover.pattern('config.json', '.'))) != 0:
        print("WARNING: You already have a config.json file. This procedure will overwrite it.")

    c = Config()

    repo_root_path = discover.ask_repo_root()

    perform_search = input("Search for 'package.json' files to automatically obtain the package root path? (y/n): ")

    if perform_search == 'y':
        print("Searching for package.json files...")
        package_jsons = list(
            map(lambda p: str(discover.make_relative(p, '.')),
                discover.package_json_recursive(repo_root_path)))
        count = len(package_jsons)

        if count == 0:
            print("Cannot find any package.json file in the repository.")
            print("You need to manually enter your package root path.")
            package_root_path = discover.ask_package_root(repo_root_path)
        elif count == 1:
            print("Found a single package.json file.")
            package_root_path = discover.directory_of_file(package_jsons[0])
        else:
            print("There are multiple package.json files in the repository.")
            package_json_path = pick_package_json(package_jsons)

            if package_json_path == -1:
                package_root_path = discover.ask_package_root(repo_root_path)
            else:
                package_root_path = discover.directory_of_file(package_json_path)
    else:
        package_root_path = discover.ask_package_root(repo_root_path)

    repo_root_path = discover.make_relative(repo_root_path, '.')
    package_root_path = discover.make_relative(package_root_path, '.')
    write_config(repo_root_path, package_root_path, c)


def write_config(repo_root_path, package_root_path, c: Config):
    repo_root_path = str(repo_root_path)
    package_root_path = str(package_root_path)
    print("Writing config.")
    print("Repo root path: " + repo_root_path)
    print("Package root path: " + package_root_path)
    c.package_root_path(package_root_path)
    c.repo_root_path(repo_root_path)
    c.save()


def pick_package_json(package_jsons):
    print("Pick package.json")
    i = 0

    for package_json in package_jsons:
        print(str(i) + '.\t' + package_json)
        i += 1

    opt = input("Enter the index of the file you wish to use, or leave empty to select manually via a dialog: ")

    if opt == '':
        return -1

    return package_jsons[int(opt)]


if __name__ == '__main__':
    main()
    input("Press any key to exit.")



import json
import os


class Config:
    def __init__(self):
        if self.exists():
            print("Loading the config.json")
            self.read()
        else:
            print("config.json file doesn't exist")
            self.config = {}

    def package_root_path(self, value=None):
        if value is not None:
            self.config["package_root_path"] = value
        else:
            return self.config["package_root_path"]

    def repo_root_path(self, value=None):
        if value is not None:
            self.config["repo_root_path"] = value
        else:
            return self.config["repo_root_path"]

    def refresh(self):
        self.read()

    def read(self):
        with open("config.json", "r") as config_file:
            self.config = json.load(config_file)

    def save(self):
        with open("config.json", "w+") as config_file:
            config_file.write(json.dumps(self.config, indent=4, sort_keys=True))

    def exists(self):
        return os.path.exists("config.json")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="git-upm-publisher",
    version="0.0.5",
    author="starikcetin <cetinsamedtarik@gmail.com> (https://github.com/starikcetin)",
    author_email="cetinsamedtarik@gmail.com",
    description="Automate UPM package releases for Git repositories containing Unity plugins.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/starikcetin/git-upm-publisher",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'gup = git_upm_publisher.main:main',
            'gup-main = git_upm_publisher.main:main',
            'gup-config-maker = git_upm_publisher.config_maker:main',
            'gup-package-json-creator = git_upm_publisher.package_json_creator:main',
            'gup-package-json-updater = git_upm_publisher.package_json_updater:main',
            'gup-upm-publisher = git_upm_publisher.upm_publisher:main',
        ],
    },
)



from git_upm_publisher.utils.config_reader import Config
from git_upm_publisher.utils.package_manager import PackageManager


def main():
    config = Config()

    pm = PackageManager(config.package_root_path())
    package_json_exists = pm.exists()

    if package_json_exists:
        print("WARNING: You already have a package.json, this procedure will overwrite it!")

    pm.create()

    print("Done. File location: " + pm.package_json_path)
    print("Don't forget to generate the meta file for the package.json by switching to Unity Editor.")


if __name__ == '__main__':
    main()
    input("Press any key to exit.")



class PackageJsonObj:
    def init_with_values(self, package_name: str, display_name: str, unity_min_version: str, description: str,
                         version: str, dependencies: dict):
        self.name = package_name
        self.displayName = display_name

        if unity_min_version != "":
            self.unity = unity_min_version

        self.description = description
        self.version = version
        self.dependencies = dependencies

    def init_from_dict(self, init_dict: dict):
        for key in init_dict:
            setattr(self, key, init_dict[key])

    def get(self, name):
        return getattr(self, name)

    def set(self, name, value):
        setattr(self, name, value)


from pathlib import Path
from git import Repo
import os
from datetime import datetime
import subprocess


class Git:
    def __init__(self, repo_root_path):
        self.repo_root_path = repo_root_path
        self.dotgit_path = os.path.join(repo_root_path, ".git/")
        assert os.path.exists(self.dotgit_path), "Cannot find the .git folder (are you sure the repo root is correct in the config.json?)"
        self.repo = Repo(repo_root_path)

    def is_dirty(self):
        return self.repo.is_dirty()

    def tag(self, commit, name, message):
        print("Tagging")
        self.repo.git.tag("-a", name, commit, "-m", message)

    def commit_all(self, message):
        print("Comitting all")
        self.repo.git.add("--all")
        self.repo.git.commit("-m", "\"" + message + "\"")

    def publish(self, package_root_path, commit_message, branch_name, version_tag):
        print("Publishing")
        print("branch before publish: " + self.repo.active_branch.name)
        
        package_root_path = Path(package_root_path).absolute()

        original_cwd = os.getcwd()
        print("Changing CWD...")
        os.chdir(self.repo_root_path)
        print("CWD: " + os.getcwd())
        subprocess.run(["npm", "install", "-g", "git-snapshot@1.1.2"], shell=True)
        subprocess.run(["git", "snapshot", "--prefix=" + package_root_path.as_posix() + "", "--message=\'" + commit_message + "\'", "--branch=" + branch_name], shell=True)
        os.chdir(original_cwd)

        # self.repo.git.snapshot("--prefix=" + package_root_path.as_posix() + "", "--message=\'" + commit_message + "\'", "--branch=" + branch_name)
        
        print("branch after publish: " + self.repo.active_branch.name)
        self.tag(branch_name, version_tag, version_tag)

    def soft_reset_last_commit(self):
        print("Soft resetting last commit")
        self.repo.git.reset("--soft", "HEAD~1")

    def clean(self):
        print("Cleaning")
        print("Stashing everything (just in case)")
        stash_name = "backup stash before repo clean by git-upm-publisher " + str(datetime.now())
        self.repo.git.stash("save", stash_name)
        print("Stash name: " + stash_name)

        if self.is_dirty():
            print("Still dirty after stash, doing a hard reset")
            self.repo.git.reset("--hard")

    def push_all(self):
        print("Pushing")
        self.repo.git.push("--all")
        self.repo.git.push("--tags")

    def status(self):
        print("Status")
        return self.repo.git.status()

    def fetch(self):
        print("Fetching")
        self.repo.git.fetch("--all")
